@@ -3,7 +3,7 @@ import json
 import logging
 import math
 import os
-from typing import Optional, Tuple, Union
+from typing import Dict, Optional, Tuple, Union
 
 import numpy as np
 import scipy
@@ -95,7 +95,7 @@ EMPHASIS = np.array(
 ) * (7.5 / 9)
 
 
-def read_gtf_file(gtf_file):
+def read_gtf_file(gtf_file: str) -> Dict:
     """Read a gammatone filterbank file.
 
     List data is converted into numpy arrays.
@@ -113,17 +113,23 @@ def read_gtf_file(gtf_file):
     return data
 
 
-def firwin2(n, f, a, window=None, antisymmetric=None):  # pylint: disable=W0613
+def firwin2(
+    n_taps: int,
+    frequencies: np.ndarray,
+    filter_gains: np.ndarray,
+    window: Union[None, str, tuple] = None,
+    antisymmetric: Optional[bool] = None,
+):  # pylint: disable=W0613
     """FIR filter design using the window method.
 
     Partial implementation of scipy firwin2 but using our own MATLAB-derived fir2.
 
     Args:
-        n (int): The number of taps in the FIR filter.
-        f (ndarray): The frequency sampling points. 0.0 to 1.0 with 1.0 being Nyquist.
-        a (ndarray): The filter gains at the frequency sampling points.
+        n_taps (int): The number of taps in the FIR filter.
+        frequencies (ndarray): The frequency sampling points. 0.0 to 1.0 with 1.0 being Nyquist.
+        filter_gains (ndarray): The filter gains at the frequency sampling points.
         window (string or (string, float), optional): See scipy.firwin2 (default: (None))
-        _antisymmetric (bool, optional): Unused but present to main compatability
+        antisymmetric (bool, optional): Unused but present to main compatability
             with scipy firwin2.
 
     Returns:
@@ -131,141 +137,153 @@ def firwin2(n, f, a, window=None, antisymmetric=None):  # pylint: disable=W0613
 
     """
     window_shape = None
+    window_type = None
     if isinstance(window, tuple):
         window_type, window_param = window if window is not None else (None, 0)
     else:
         window_type = window
 
-    order = n - 1
+    order = n_taps - 1
 
     if window_type == "kaiser":
-        window_shape = scipy.signal.kaiser(n, window_param)
+        window_shape = scipy.signal.kaiser(n_taps, window_param)
 
     if window_shape is None:
-        b, _ = fir2(order, f, a)
+        filter_coef, _ = fir2(order, frequencies, filter_gains)
     else:
-        b, _ = fir2(order, f, a, window_shape)
+        filter_coef, _ = fir2(order, frequencies, filter_gains, window_shape)
 
-    return b
+    return filter_coef
 
 
 def fir2(
-    nn: int, ff: np.ndarray, aa: np.ndarray, npt: Optional[int] = None
+    filter_length: int,
+    frequencies: np.ndarray,
+    filter_gains: np.ndarray,
+    n_interpolate: Optional[int] = None,
 ) -> Tuple[np.ndarray, int]:
     """FIR arbitrary shape filter design using the frequency sampling method.
 
     Translation of MATLAB fir2.
 
     Args:
-        nn (int): Order
-        ff (ndarray): Frequency breakpoints (0 < F < 1) where 1 is Nyquist rate.
-                        First and last elements must be 0 and 1 respectively
-        aa (ndarray): Magnitude breakpoints
-        npt (int, optional): Number of points for freq response interpolation
-            (default: max (smallest power of 2 greater than nn, 512))
+        filter_length (int): Order
+        frequencies (ndarray): The frequency sampling points (0 < frequencies < 1) where 1 is Nyquist rate.
+                        First and last elements must be 0 and 1 respectively.
+        filter_gains (ndarray): The filter gains at the frequency sampling points.
+        n_interpolate (int, optional): Number of points for freq response interpolation
+            (default: max(smallest power of 2 greater than nn, 512))
 
     Returns:
-        ndarray: nn + 1 filter coefficients, 1
+        np.ndarray: nn + 1 filter coefficients, 1
 
     """
     # Work with filter length instead of filter order
-    nn += 1
+    filter_length += 1
 
-    if npt is None:
-        new_npt: int = 2 ** np.ceil(math.log(nn) / math.log(2)) if nn >= 1024 else 512
-        wind = scipy.signal.hamming(nn)
+    if n_interpolate is None:
+        wind = scipy.signal.hamming(filter_length)
     else:
-        wind = npt
-        new_npt = 2 ** np.ceil(math.log(nn) / math.log(2)) if nn >= 1024 else 512
-    npt = new_npt
+        wind = n_interpolate
+    n_interpolate = (
+        2 ** np.ceil(math.log(filter_length) / math.log(2))
+        if filter_length >= 1024
+        else 512
+    )
 
-    lap = np.fix(npt / 25).astype(int)
+    lap = np.fix(n_interpolate / 25).astype(int)
 
-    nbrk = max(len(ff), len(aa))
+    nbrk = max(len(frequencies), len(filter_gains))
 
-    ff[0] = 0
-    ff[nbrk - 1] = 1
+    frequencies[0] = 0
+    frequencies[nbrk - 1] = 1
 
-    H = np.zeros(npt + 1)
+    H = np.zeros(n_interpolate + 1)
     nint = nbrk - 1
-    df = np.diff(ff, n=1)
+    df = np.diff(frequencies, n=1)
 
-    npt += 1
+    n_interpolate += 1
     nb = 0
-    H[0] = aa[0]
+    H[0] = filter_gains[0]
 
     for i in np.arange(nint):
         if df[i] == 0:
             nb = int(np.ceil(nb - lap / 2))
             ne: int = nb + lap - 1
         else:
-            ne = int(np.fix(ff[i + 1] * npt)) - 1
+            ne = int(np.fix(frequencies[i + 1] * n_interpolate)) - 1
 
         j = np.arange(nb, ne + 1)
         inc: Union[float, np.ndarray] = 0.0 if nb == ne else (j - nb) / (ne - nb)
-        H[nb : (ne + 1)] = inc * aa[i + 1] + (1 - inc) * aa[i]
+        H[nb : (ne + 1)] = inc * filter_gains[i + 1] + (1 - inc) * filter_gains[i]
         nb = ne + 1
 
-    dt = 0.5 * (nn - 1)
-    rad = -dt * 1j * math.pi * np.arange(0, npt) / (npt - 1)
+    dt = 0.5 * (filter_length - 1)
+    rad = -dt * 1j * math.pi * np.arange(0, n_interpolate) / (n_interpolate - 1)
     H = H * np.exp(rad)
 
-    H = np.concatenate((H, H[npt - 2 : 0 : -1].conj()))
+    H = np.concatenate((H, H[n_interpolate - 2 : 0 : -1].conj()))
     ht = np.real(np.fft.ifft(H))
 
-    b = ht[0:nn] * wind
+    b = ht[0:filter_length] * wind
 
     return b, 1
 
 
 def gen_tone(
-    freq: float, duration: float, fs: float = 44100.0, level: float = 0.0
+    freq: float, duration: float, sample_frequency: float = 44100.0, level: float = 0.0
 ) -> np.ndarray:
     """Generate a pure tone.
 
     Args:
         freq (float): Frequency of tone in Hz.
         duration (float): Duration of tone in seconds.
-        fs (float, optional): Sample rate of generated tone in Hz. (default: 44100)
+        sample_frequency (float, optional): Sample rate of generated tone in Hz. (default: 44100)
         level (float, optional): Level of tone in dB SPL. (default: 0)
+
+    Returns:
+        np.ndarray
     """
     return (
         1.4142
         * np.power(10, (0.05 * level))
-        * np.sin(2 * np.pi * freq * np.arange(1, duration * fs + 1) / fs)
+        * np.sin(
+            2
+            * np.pi
+            * freq
+            * np.arange(1, duration * sample_frequency + 1)
+            / sample_frequency
+        )
     )
 
 
 def gen_eh2008_speech_noise(
     duration: float,
-    fs: float = 44100.0,
+    sample_frequency: float = 44100.0,
     level: Optional[float] = None,
     supplied_b: Optional[np.ndarray] = None,
 ) -> np.ndarray:
     """Generate speech shaped noise.
 
-    Start with white noise and re-shape to ideal SII, ie flat to 500 Hz, and
-    sloping -9db/oct beyond that.
+    Start with white noise and re-shape to ideal SII, ie flat to 500 Hz, and sloping -9db/oct beyond that.
 
-    Slightly different shape from SII stylised same as
-    EarHEar 2008 paper, Moore et al.
+    Slightly different shape from SII stylised same as EarHEar 2008 paper, Moore et al.
 
     Args:
         duration (float): Duration of signal in seconds
-        fs (float): Sampling rate
+        sample_frequency (float): Sampling rate
         level (float, optional): Normalise to level dB if present
-        supplied_b (ndarray, optional): High-pass filter
-            (default: uses built-in pre-emphasis filter)
+        supplied_b (ndarray, optional): High-pass filter (default: uses built-in pre-emphasis filter)
 
     Returns:
         ndarray: Noise signal
 
     """
-    fs = int(fs)
-    n_samples = int(duration * fs)
+    sample_frequency = int(sample_frequency)
+    n_samples = int(duration * sample_frequency)
 
     # this rescales so that we get -7.5 dB/oct up to 8kHz, and -13 dB/oct above that
-    norm_freq = GEN_NOISE_HZ / (fs / 2)
+    norm_freq = GEN_NOISE_HZ / (sample_frequency / 2)
     last_f_idx = np.max(np.where(norm_freq < 1))
     norm_freq = np.append(norm_freq[0 : last_f_idx + 1], 1)
 
@@ -275,7 +293,7 @@ def gen_eh2008_speech_noise(
     m = np.exp(np.log(10) * norm_emph / 20)
 
     # Create type II filter with 10 msec window and even number of taps
-    n_taps = int(2 * np.ceil(10 * (fs / 2000))) + 1
+    n_taps = int(2 * np.ceil(10 * (sample_frequency / 2000))) + 1
     b = (
         supplied_b
         if supplied_b is not None
@@ -283,22 +301,26 @@ def gen_eh2008_speech_noise(
     )
 
     # white noise, 0 DC
-    nburst = np.random.random((1, n_samples + len(b))) - 0.5
+    n_burst = np.random.random((1, n_samples + len(b))) - 0.5
 
     # remove low-freq noise that may bias RMS estimate, -33dB at 50 Hz
-    eh2008_nse = scipy.signal.lfilter(b, 1, nburst)
+    eh2008_nse = scipy.signal.lfilter(b, 1, n_burst)
 
     # high-pass filter to remove low freqs (will be 2-pass with filtfilt)
-    hpf = scipy.signal.ellip(3, 0.1, 50, 100 / (fs / 2), "high")
-    padlen = 3 * (max(len(hpf[1]), len(hpf[0])) - 1)
-    eh2008_nse = scipy.signal.filtfilt(*hpf, eh2008_nse, padlen=padlen).flatten()
+    high_pass_filter = scipy.signal.ellip(
+        3, 0.1, 50, 100 / (sample_frequency / 2), "high"
+    )
+    padlen = 3 * (max(len(high_pass_filter[1]), len(high_pass_filter[0])) - 1)
+    eh2008_nse = scipy.signal.filtfilt(
+        *high_pass_filter, eh2008_nse, padlen=padlen
+    ).flatten()
 
     # this introduces a delay so remove it, ie time-ADVANCE audio
     # compensating shift to time-align all filter outputs
-    dly_shift = int(np.floor(len(b) / 2))
-    valid_len = int(np.size(eh2008_nse) - dly_shift)  # _advance_ filter outputs
+    delay_shift = int(np.floor(len(b) / 2))
+    valid_len = int(np.size(eh2008_nse) - delay_shift)  # _advance_ filter outputs
     # time advance
-    eh2008_nse[0:valid_len] = eh2008_nse[dly_shift:]
+    eh2008_nse[0:valid_len] = eh2008_nse[delay_shift:]
     eh2008_nse = eh2008_nse[0:n_samples]
 
     if level is not None:
@@ -312,9 +334,9 @@ def gen_eh2008_speech_noise(
 
 
 def generate_key_percent(
-    sig: np.ndarray,
-    thr_dB: float,
-    winlen: int,
+    signal: np.ndarray,
+    threshold_db: float,
+    window_length: int,
     percent_to_track: Optional[float] = None,
 ) -> Tuple[np.ndarray, float]:
     """Generate key percent.
@@ -323,28 +345,32 @@ def generate_key_percent(
     of rms, adaptively sets threshold after looking at histogram of whole recording
 
     Args:
-        sig (ndarray): The signal to analyse
-        thr_dB (float): fixed energy threshold (dB)
-        winlen (int): length of window in samples
-        percent_to_track (float, optional): Track a percentage of frames (default: {None})
+        signal (ndarray): The signal to analyse.
+        threshold_db (float): fixed energy threshold (dB).
+        window_length (int): length of window in samples.
+        percent_to_track (float, optional): Track a percentage of frames (default: {None}).
 
     Raises:
-        ValueError: percent_to_track is set too high
+        ValueError: percent_to_track is set too high.
 
     Returns:
-        (ndarray, float) -- "key" and rms threshold
-            The key array of indices of samples used in rms calculation,
+        (tuple): containig
+        - key (ndarray): The key array of indices of samples used in rms calculation.
+        - used_threshold_db (float): Root Mean Squared threshold.
+
             and the threshold used to get a more accurate rms calculation
     """
-    winlen = int(winlen)
-    sig = sig.flatten()
-    if winlen != math.floor(winlen):  # whoops on fractional indexing: 7-March 2002
-        winlen = math.floor(winlen)
-        logging.warning("Window length must be integer: now %s", winlen)
+    window_length = int(window_length)
+    signal = signal.flatten()
+    if window_length != math.floor(
+        window_length
+    ):  # whoops on fractional indexing: 7-March 2002
+        window_length = math.floor(window_length)
+        logging.warning(f"Window length must be integer: now {window_length}")
 
-    siglen = len(sig)
+    signal_length = len(signal)
 
-    expected = thr_dB
+    expected = threshold_db
     # new Dec 2003. Possibly track percentage of frames rather than fixed threshold
     if percent_to_track is not None:
         logging.info("tracking %s percentage of frames", percent_to_track)
@@ -354,28 +380,32 @@ def generate_key_percent(
     # put floor into histogram distribution
     non_zero = np.power(10, (expected - 30) / 10)
 
-    nframes = -1
-    totframes = math.floor(siglen / winlen)
-    every_dB = np.zeros(totframes)
+    n_frames = -1
+    total_frames = math.floor(signal_length / window_length)
+    every_db = np.zeros(total_frames)
 
-    for ix in np.arange(0, winlen * totframes - 1, winlen):
-        nframes += 1
-        this_sum = np.sum(np.power(sig[ix : (ix + winlen)].astype("float"), 2))
-        every_dB[nframes] = 10 * np.log10(non_zero + this_sum / winlen)
-    nframes += 1
+    for ix in np.arange(
+        0, window_length * total_frames - 1, window_length
+    ):  # pylint: disable=invalid-name
+        n_frames += 1
+        this_sum = np.sum(
+            np.power(signal[ix : (ix + window_length)].astype("float"), 2)
+        )
+        every_db[n_frames] = 10 * np.log10(non_zero + this_sum / window_length)
+    n_frames += 1
 
     # from now on save only those analysed
-    every_dB = every_dB[:nframes]
+    every_db = every_db[:n_frames]
 
     # Bec 2003, was 100 to give about a 0.5 dB quantising of levels
-    n_bins, levels = np.histogram(every_dB, 140)
+    n_bins, levels = np.histogram(every_db, 140)
     if percent_to_track is not None:
         # min number of bins to use
-        inactive_bins = (100 - percent_to_track) * nframes / 100
+        inactive_bins = (100 - percent_to_track) * n_frames / 100
         n_levels = len(levels)
         inactive_ix = 0
         ix_count = 0
-        for ix in np.arange(0, n_levels, 1):
+        for ix in np.arange(0, n_levels, 1):  # pylint: disable=invalid-name
             inactive_ix = inactive_ix + n_bins[ix]
             if inactive_ix > inactive_bins:
                 break
@@ -387,31 +417,35 @@ def generate_key_percent(
         expected = levels[max(1, ix_count)]
 
     # set new threshold conservatively to include more bins than desired
-    used_thr_dB = expected
+    used_threshold_db = expected
 
     # histogram should produce a two-peaked curve: thresh should be set in valley
     # between the two peaks, and set threshold a bit above that,
     # as it heads for main peak
-    frame_index = np.nonzero(every_dB >= expected)[0]
+    # FixMe : Could Otsu's method (from image processing) be used here? https://en.wikipedia.org/wiki/Otsu's_method
+    frame_index = np.nonzero(every_db >= expected)[0]
     valid_frames = len(frame_index)
-    key = np.zeros((1, valid_frames * winlen))[0]
+    key = np.zeros((1, valid_frames * window_length))[0]
 
     # convert frame numbers into indices for sig
-    for ix in np.arange(valid_frames):
+    for ix in np.arange(valid_frames):  # pylint: disable=invalid-name
         meas_span = np.arange(
-            (frame_index[ix] * winlen), (frame_index[ix] + 1) * winlen
+            (frame_index[ix] * window_length), (frame_index[ix] + 1) * window_length
         )
-        key_span = np.arange(((ix) * winlen), (ix + 1) * winlen, 1)
+        key_span = np.arange(((ix) * window_length), (ix + 1) * window_length, 1)
         key[key_span] = meas_span
         key = key.flatten()
 
-    return key, used_thr_dB
+    return key, used_threshold_db
 
 
 def measure_rms(
-    signal: np.ndarray, fs: float, dB_rel_rms: float, percent_to_track: float = None
+    signal: np.ndarray,
+    sample_frequency: float,
+    db_rel_rms: float,
+    percent_to_track: Optional[float] = None,
 ) -> tuple:
-    """Measure rms.
+    """Measure Root Mean Square.
 
     A sophisticated method of measuring RMS in a file. It splits the signal up into
     short windows, performs  a histogram of levels, calculates an approximate RMS,
@@ -420,27 +454,30 @@ def measure_rms(
     threshold.
 
     Args:
-        signal (ndarray): the signal of which to measure the rms
-        fs (float): sampling frequency
-        dB_rel_rms (float): threshold for frames to track
+        signal (ndarray): the signal of which to measure the Root Mean Square.
+        sample_frequency (float): sampling frequency.
+        db_rel_rms (float): threshold for frames to track.
         percent_to_track (float, optional): track percentage of frames,
             rather than threshold (default: {None})
     Returns:
         (tuple): tuple containing
         - rms (float): overall calculated rms (linear)
         - key (ndarray): "key" array of indices of samples used in rms calculation
-        - rel_dB_thresh (float): fixed threshold value of -12 dB
+        - rel_db_thresh (float): fixed threshold value of -12 dB
         - active (float): proportion of values used in rms calculation
     """
-    fs = int(fs)
+    sample_frequency = int(sample_frequency)
     # first RMS is of all signal.
     first_stage_rms = np.sqrt(np.sum(np.power(signal, 2) / len(signal)))
     # use this RMS to generate key threshold to get more accurate RMS
-    key_thr_dB = max(20 * np.log10(first_stage_rms) + dB_rel_rms, -80)
+    key_thr_db = max(20 * np.log10(first_stage_rms) + db_rel_rms, -80)
 
-    # move key_thr_dB to account for noise less peakier than signal
-    key, used_thr_dB = generate_key_percent(
-        signal, key_thr_dB, round(WIN_SECS * fs), percent_to_track=percent_to_track
+    # move key_thr_db to account for noise less peakier than signal
+    key, used_thr_db = generate_key_percent(
+        signal,
+        key_thr_db,
+        round(WIN_SECS * sample_frequency),
+        percent_to_track=percent_to_track,
     )
 
     idx = key.astype(int)  # move into generate_key_percent
@@ -448,9 +485,9 @@ def measure_rms(
     # (for independent==1 loop where it sets a target for rms measure)
     active = 100 * len(key) / len(signal)
     rms = np.sqrt(np.sum(np.power(signal[idx], 2)) / len(key))
-    rel_dB_thresh = used_thr_dB - 20 * np.log10(rms)
+    rel_db_thresh = used_thr_db - 20 * np.log10(rms)
 
-    return rms, idx, rel_dB_thresh, active
+    return rms, idx, rel_db_thresh, active
 
 
 def pad(signal, length):
@@ -500,40 +537,47 @@ def read_signal(
     if offset != 0:
         wave_file.seek(offset)
 
-    x = wave_file.read(frames=nsamples)
+    signal = wave_file.read(frames=nsamples)
 
     if wave_file.samplerate != MSBG_FS:
-        x = scipy.signal.resample(x, int(MSBG_FS * x.shape[0] / wave_file.samplerate))
+        signal = scipy.signal.resample(
+            signal, int(MSBG_FS * signal.shape[0] / wave_file.samplerate)
+        )
 
-    return x
+    return signal
 
 
 def write_signal(
-    filename: str, x: np.ndarray, fs: float, floating_point: bool = True
+    filename: str,
+    signal: np.ndarray,
+    sample_frequency: float,
+    floating_point: bool = True,
 ) -> None:
     """Write a signal as fixed or floating point wav file.
 
     Args:
-        filename (str): name of file in to write to
-        x (ndarray): signal to write
-        fs (float): sampling frequency
-        floating_point (bool): write as floating point else an ints (default: True)
+        filename (str): name of file in to write to.
+        signal (ndarray): signal to write.
+        sample_frequency (float): sampling frequency.
+        floating_point (bool): write as floating point else an ints (default: True).
     """
 
-    if fs != MSBG_FS:
-        logging.warning("Sampling rate mismatch: %s with sr=%s.", filename, fs)
+    if sample_frequency != MSBG_FS:
+        logging.warning(
+            f"Sampling rate mismatch: {filename} with sample frequency = {sample_frequency}."
+        )
         # raise ValueError("Sampling rate mismatch")
 
     if floating_point is False:
         if TEST_NBITS == 16:
             subtype = "PCM_16"
             # If signal is float and we want int16
-            x *= 32768
-            x = x.astype(np.dtype("int16"))
-            assert np.max(x) <= 32767 and np.min(x) >= -32768
+            signal *= 32768
+            signal = signal.astype(np.dtype("int16"))
+            assert np.max(signal) <= 32767 and np.min(signal) >= -32768
         elif TEST_NBITS == 24:
             subtype = "PCM_24"
     else:
         subtype = "FLOAT"
 
-    soundfile.write(filename, x, fs, subtype=subtype)
+    soundfile.write(filename, signal, sample_frequency, subtype=subtype)
